/* $Id: StateStorage.java,v 1.4 2006/05/30 19:20:48 gsson Exp $ */

/*
 * Copyright (c) 2006 Henrik Gustafsson <henrik.gustafsson@fnord.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package se.fnord.ecalc;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import se.fnord.domhelper.DomHelper;
import se.fnord.domhelper.DomHelperException;
import se.fnord.ecalc.parser.ExpressionException;
import se.fnord.ecalc.parser.ExpressionParser;
import se.fnord.ecalc.parser.IValue;
import se.fnord.ecalc.parser.Literal;
import se.fnord.ecalc.parser.ParseException;
import se.fnord.ecalc.parser.SimpleNode;
import se.fnord.ecalc.parser.StringBuilderContext;
import se.fnord.ecalc.parser.StringBuilderVisitor;
import se.fnord.ecalc.parser.functions.UserFunction;

public class StateStorage {
	Document document = null;
	DomHelper helper = null;

	public StateStorage() {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.newDocument();
			helper = new DomHelper(document);
			createHierarchy();

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public StateStorage(File file) {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			try {
				document = builder.parse(file);
			} catch (FileNotFoundException e) {
				document = builder.newDocument();
			} catch (SAXException e) {
				document = builder.newDocument();
			} catch (IOException e) {
				document = builder.newDocument();
			}

			helper = new DomHelper(document);
			if (helper.nextByTagName("state"))
				helper.enter();
			else {
				createHierarchy();
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DomHelperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void save(File file) {
		try {
			DOMSource source = new DOMSource(document);
			StreamResult result = new StreamResult(new FileOutputStream(file));

			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.transform(source, result);
		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerFactoryConfigurationError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void createHierarchy() {
		try {
			helper.root();
			helper.clear();
			helper.insert("state");
			helper.enter();
			helper.insert("answer");
			helper.insert("variables");
			helper.insert("functions");
			helper.insert("history");
			helper.insert("calculations");
			helper.begin();
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}
	}

	private void setAsExpression(SimpleNode expression) {

		try {
			StringBuilderVisitor v = new StringBuilderVisitor();
			StringBuilderContext c = new StringBuilderContext();
			expression.accept(v, c);
			helper.setText(c.getString());
		} catch (ExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DomHelperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private SimpleNode getAsExpression() {
		try {
			ExpressionParser parser = new ExpressionParser(new StringReader(
					helper.getText()));
			SimpleNode parseTree = parser.Start();
			return parseTree;
		} catch (ExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DomHelperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	private void setAsIValue(IValue v) {
		try {
			switch (v.getPreferredType()) {
			case IValue.TYPE_LONG:
				helper.setAttribute("type", "long");
				break;
			case IValue.TYPE_DOUBLE:
				helper.setAttribute("type", "double");
				break;
			default:
				return;
			}
			helper.setText(v.toString());

		} catch (ExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DomHelperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private IValue getAsIValue() {
		try {
			String type;
			if (helper.hasAttribute("type")) {
				type = helper.getAttribute("type"); 
				if (type.equals("double")) {
					Literal l = new Literal();
					l.fromDouble(Double.parseDouble(helper.getText()));
					return l;
				}
				else if (type.equals("long")) {
					Literal l = new Literal();
					l.fromLong(Long.parseLong(helper.getText()));
					return l;
				}
			}
			else {
				return new Literal(0);
			}
		} catch (ExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DomHelperException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return new Literal(0);
	}

	public void setHistory(List history) {
		try {
			helper.begin();
			if (helper.nextByTagName("history")) {
				helper.enter();
				helper.clear();

				ListIterator i = history.listIterator();
				while (i.hasNext()) {
					String c = (String) i.next();
					helper.insert("entry");
					helper.setText(c);
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}
	}

	public List getHistory() {
		LinkedList history = new LinkedList();
		try {
			helper.begin();
			if (helper.nextByTagName("history")) {
				helper.enter();

				while (helper.next()) {
					history.add(helper.getText());
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}
		return history;
	}

	public void setCalculations(List calculations) {
		try {
			helper.begin();
			if (helper.nextByTagName("calculations")) {
				helper.enter();
				helper.clear();
	
				ListIterator i = calculations.listIterator();
				while (i.hasNext()) {
					Calculation c = (Calculation) i.next();
	
					helper.insert("calculation");
					helper.enter();
					helper.insert("exp");
					helper.setText(c.getExpression());
					helper.insert("res");
					helper.setText(c.getResult());
					helper.leave();
				}
	
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}

	}

	public List getCalculations() {
		LinkedList calculations = new LinkedList();
		try {
			helper.begin();
			if (helper.nextByTagName("calculations")) {
				helper.enter();
	
				while (helper.nextByTagName("calculation")) {
					String result;
					String expression;
					helper.enter();
					if (!helper.nextByTagName("exp"))
						continue;
	
					expression = helper.getText();
	
					if (!helper.nextByTagName("res"))
						continue;
	
					result = helper.getText();
	
					helper.leave();
	
					Calculation c = new Calculation(expression, result);
					calculations.add(c);
	
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}

		return calculations;
	}

	public void setAnswer(IValue ans) {
		helper.begin();
		if (helper.nextByTagName("answer")) {
			setAsIValue(ans);
		}
	}

	public IValue getAnswer() {
		helper.begin();
		if (helper.nextByTagName("answer")) {
			return getAsIValue();
		}
		return new Literal(0);
	}

	public void setVariables(Map variables) {
		try {
			helper.begin();
			if (helper.nextByTagName("variables")) {	
				helper.enter();
				helper.clear();

				Set names = variables.keySet();
	
				Iterator i = names.iterator();
				while (i.hasNext()) {
					helper.insert("variable");
					String name = (String) i.next();
					IValue value = (IValue) variables.get(name);
					setAsIValue(value);
					helper.setAttribute("name", name);
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}
	}

	public Map getVariables() {
		Map variables = new HashMap();
		try {
			helper.begin();
			if (helper.nextByTagName("variables")) {
	
				helper.enter();
	
				while (helper.nextByTagName("variable")) {
					variables.put(helper.getAttribute("name"), getAsIValue());
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}

		return variables;
	}

	public void setFunctions(Map functions) {
		try {
			helper.begin();
			if (helper.nextByTagName("functions")) {	
				helper.enter();
				helper.clear();

				Set names = functions.keySet();
	
				Iterator i = names.iterator();
				while (i.hasNext()) {
	
					String name = (String) i.next();
					Object o = functions.get(name);
					if (o instanceof UserFunction) {
						helper.insert("function");
						SimpleNode expression = ((UserFunction) o).getExpression();
						setAsExpression(expression);
						helper.setAttribute("params", ((UserFunction) o)
								.getParamNames());
						helper.setAttribute("name", name);
					}
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}

	}

	public Map getFunctions() {
		Map functions = new HashMap();
		try {
			helper.begin();
			if (helper.nextByTagName("functions")) {
	
				helper.enter();
	
				while (helper.nextByTagName("function")) {
					UserFunction f = new UserFunction(
							helper.getAttribute("params"), getAsExpression());
					functions.put(helper.getAttribute("name"), f);
				}
				helper.leave();
			}
		} catch (DomHelperException e) {
			// TODO: handle exception
			e.printStackTrace();
		}

		return functions;
	}

	public void dump() {
		try {
			DOMSource source = new DOMSource(document);
			StreamResult result = new StreamResult(System.out);

			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();

			transformer.transform(source, result);
		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerFactoryConfigurationError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
