/* $Id: EvaluationContext.java,v 1.2 2006/05/30 21:18:17 gsson Exp $ */

/*
 * Copyright (c) 2006 Henrik Gustafsson <henrik.gustafsson@fnord.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package se.fnord.ecalc.parser;

import java.util.HashMap;
import java.util.Map;

public class EvaluationContext implements ExpressionParserContext {
	
	protected Map variables = new HashMap();
	protected Map functions = new HashMap();
	protected Map constants = new HashMap();
	protected Literal answer = new Literal();
	
	public int arithmeticType = IValue.TYPE_AUTO;
	public int angleType = IOperators.ANGLE_RAD;

	
	
	public void setAnswer(IValue v) {
		try {
			answer.fromIValue(v);
		} catch (ExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public Literal getAnswer() {
		return answer;
	}
	
	public Map getFunctions() {
		return functions;
	}

	public void setFunctions(Map functions) {
		this.functions = functions;
	}
	
	public Map getVariables() {
		return variables;
	}

	public void setVariables(Map variables) {
		this.variables = variables;
	}
	
	
	public IFunction getFunction(String name) throws ExpressionException {
		IFunction f = (IFunction)functions.get(name);
		if (f == null)
			throw new ExpressionException("%se.fnord.ecalc.exceptions.expressionexception.undefined_function");
		return f;
	}
	
	public void setFunction(String name, IFunction f) {
		functions.put(name, f);
	}
	
	protected Literal getConstant(String name) throws ExpressionException {
		return (Literal)constants.get(name);
	}
	
	protected Literal getVariable(String name) throws ExpressionException {
		return (Literal)variables.get(name);
	}
	
	public Literal getSymbol(String name) throws ExpressionException {
		if (name.equals("ans"))
			return getAnswer();
		
		Literal l = (Literal)constants.get(name);
		if (l == null) {
			l = (Literal)variables.get(name);
			if (l == null) {
				l = new Literal();
				variables.put(name, l);
			}
		}
		return l;
	}
	
	public void setConstant(String name, Literal l) throws ExpressionException {
		constants.put(name, new Constant(l));
	}	
	
	public void setVariable(String name, Literal l) throws ExpressionException {
		// TODO: Verify this. It's late. I have no idea what I'm doing.
		Literal l2 = (Literal)variables.get(name);
		if (l2 == null) {
			variables.put(name, l);
		}
		else {
			l2.fromIValue(l);
		}
	}

}
