/* $Id: ExpressionParserTest.java,v 1.6 2006/05/30 19:20:48 gsson Exp $ */

/*
 * Copyright (c) 2006 Henrik Gustafsson <henrik.gustafsson@fnord.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package se.fnord.ecalc.parser;

import se.fnord.ecalc.parser.functions.Cosine;
import se.fnord.ecalc.parser.functions.Round;
import se.fnord.ecalc.parser.functions.Sine;

public class ExpressionParserTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		ExpressionParser parser = new ExpressionParser(System.in);
		EvaluationContext context = new EvaluationContext();
		context.setFunction("sin", new Sine());
		context.setFunction("cos", new Cosine());
		context.setFunction("round", new Round());
		try {

			/*
			 * Start parsing from the nonterminal "Start".
			 */
			SimpleNode parseTree = parser.Start();

			ExpressionParserVisitor visitor = new EvaluationVisitor();
			System.out.println(parseTree.accept(visitor, context));
			parseTree.dump("");
		} catch (Exception e) {

			/*
			 * An exception occurred during parsing. Print the error message on
			 * standard output.
			 */
			System.out.println("---------------------------");
			System.out.println("Sorry, couldn't parse that.");
			System.out.println(e.getMessage());
			System.out.println("---------------------------");

			/*
			 * Print the call stack
			 */
			System.out.println("Call stack:");
			e.printStackTrace();
			System.out.println("---------------------------");
		}
	}

}
