/* $Id: OverlayContext.java,v 1.3 2006/05/30 21:18:17 gsson Exp $ */

/*
 * Copyright (c) 2006 Henrik Gustafsson <henrik.gustafsson@fnord.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package se.fnord.ecalc.parser;

public class OverlayContext extends EvaluationContext {

	EvaluationContext parent = null;
	
	public OverlayContext(EvaluationContext c) {
		if (c == null)
			throw new IllegalArgumentException();
		parent = c;
	}

	public IFunction getFunction(String name) throws ExpressionException {
		IFunction f = (IFunction)functions.get(name);
		if (f == null)
			f = parent.getFunction(name);

		return f;
	}
	
	public void setFunction(String name, IFunction f) {
		functions.put(name, f);
	}
	
	public Literal getSymbol(String name) throws ExpressionException {
		Literal l = (Literal)constants.get(name);
		if (l == null) {
			l = parent.getConstant(name);
			if (l == null) {			
				l = (Literal)variables.get(name);
				if (l == null) {
					l = parent.getVariable(name);
					if (l == null) {
						l = new Literal();
						variables.put(name, l);
					}
				}
			}
		}
		return l;
	}
	
	public void setConstant(String name, Literal l) throws ExpressionException {
		constants.put(name, new Constant(l));
	}	
	
	public void setVariable(String name, Literal l) throws ExpressionException {
		// TODO: Verify this. It's late. I have no idea what I'm doing.
		Literal l2 = (Literal)variables.get(name);
		if (l2 == null) {
			variables.put(name, l);
		}
		else {
			l2.fromIValue(l);
		}
	}

}
