/* $Id: StringBuilderVisitor.java,v 1.3 2006/05/30 19:20:48 gsson Exp $ */

/*
 * Copyright (c) 2006 Henrik Gustafsson <henrik.gustafsson@fnord.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copie((StringBuilderContext) data).
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package se.fnord.ecalc.parser;


public class StringBuilderVisitor implements ExpressionParserVisitor {
	
	public IValue visit(SimpleNode node, ExpressionParserContext data)
			throws ExpressionException {
		// TODO Auto-generated method stub
		return null;
	}

	public IValue visit(ASTStart node, ExpressionParserContext data)
			throws ExpressionException {
		((SimpleNode) node.jjtGetChild(0)).accept(this, data);
		return null;
	}

	public IValue visit(ASTExpression node, ExpressionParserContext data)
			throws ExpressionException {
		if (node.parent instanceof ASTStart
				|| node.parent instanceof ASTFunction
				|| node.parent instanceof ASTAssign) {
			((SimpleNode) node.jjtGetChild(0)).accept(this, data);
			return null;
		}
		((StringBuilderContext) data).append("(");
		((SimpleNode) node.jjtGetChild(0)).accept(this, data);
		((StringBuilderContext) data).append(")");
		return null;
	}

	public IValue visit(ASTAssign node, ExpressionParserContext data)
			throws ExpressionException {
		if (node.jjtGetNumChildren() == 2 && node.jjtGetChild(0) instanceof ASTFunctionDef) {
			ASTFunctionDef fn = (ASTFunctionDef)node.jjtGetChild(0);
			((StringBuilderContext) data).append("def ");
			((StringBuilderContext) data).append(fn.name);
			((StringBuilderContext) data).append("(");
			if (fn.paramnames.length > 0) {
				((StringBuilderContext) data).append(fn.paramnames[0]);
				for (int i = 1; i < fn.paramnames.length; i++) {
					((StringBuilderContext) data).append(", ");
					((StringBuilderContext) data).append(fn.paramnames[0]);
				}
			}
			((StringBuilderContext) data).append(") { ");
			((SimpleNode)node.jjtGetChild(1)).accept(this, data);
			((StringBuilderContext) data).append(" }");
		}
		else {
			((SimpleNode) node.jjtGetChild(0)).accept(this, data);
			for (int i = 1; i < node.jjtGetNumChildren(); i++) {
				((StringBuilderContext) data).append(" = ");
				((SimpleNode) node.jjtGetChild(i)).accept(this, data);
			}
		}
		return null;
	}

	public IValue visit(ASTBinaryOperator node, ExpressionParserContext data)
			throws ExpressionException {
		((SimpleNode)node.jjtGetChild(0)).accept(this, data);
		for (int i = 1; i < node.jjtGetNumChildren(); i++) {
			switch(((Integer)node.operators.get(i - 1)).intValue()) {
			case ASTBinaryOperator.ADDITION:
				((StringBuilderContext) data).append(" + ");
				break;
			case ASTBinaryOperator.SUBTRACTION:
				((StringBuilderContext) data).append(" - ");
				break;
			case ASTBinaryOperator.MULTIPLICATION:
				((StringBuilderContext) data).append(" * ");
				break;
			case ASTBinaryOperator.DIVISION:
				((StringBuilderContext) data).append(" / ");
				break;
			case ASTBinaryOperator.MODULUS:
				((StringBuilderContext) data).append(" % ");
				break;
			case ASTBinaryOperator.POWER:
				((StringBuilderContext) data).append("^");
				break;
			case ASTBinaryOperator.SHL:
				((StringBuilderContext) data).append(" << ");
				break;
			case ASTBinaryOperator.SHR:
				((StringBuilderContext) data).append(" >> ");
				break;
			case ASTBinaryOperator.AND:
				((StringBuilderContext) data).append(" & ");
				break;
			case ASTBinaryOperator.OR:
				((StringBuilderContext) data).append(" | ");
				break;
			}
			((SimpleNode) node.jjtGetChild(i)).accept(this, data);
		}
		return null;
	}

	public IValue visit(ASTUnary node, ExpressionParserContext data)
			throws ExpressionException {
		if (node.operator == ASTUnary.NEGATE)
			((StringBuilderContext) data).append(" -");
		((SimpleNode) node.jjtGetChild(0)).accept(this, data);
		return null;
	}

	public IValue visit(ASTLiteral node, ExpressionParserContext data)
			throws ExpressionException {
		((StringBuilderContext) data).append(node.value.toString());
		return null;
	}

	public IValue visit(ASTVariable node, ExpressionParserContext data)
			throws ExpressionException {
		((StringBuilderContext) data).append(node.name);
		return null;
	}

	public IValue visit(ASTFunction node, ExpressionParserContext data)
			throws ExpressionException {
		((StringBuilderContext) data).append(node.name);
		((StringBuilderContext) data).append("(");
		if (node.jjtGetNumChildren() > 0) {
			((SimpleNode) node.jjtGetChild(0)).accept(this, data);
			for (int i = 1; i < node.jjtGetNumChildren(); i++) {
				((StringBuilderContext) data).append(", ");
				((SimpleNode) node.jjtGetChild(i)).accept(this, data);
			}
		}
		((StringBuilderContext) data).append(")");

		return null;
	}

	public IValue visit(ASTFunctionDef node, ExpressionParserContext data)
			throws ExpressionException {
		return null;
	}

}
