/* $Id: UserFunction.java,v 1.4 2006/05/30 21:18:17 gsson Exp $ */

/*
 * Copyright (c) 2006 Henrik Gustafsson <henrik.gustafsson@fnord.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package se.fnord.ecalc.parser.functions;

import se.fnord.ecalc.parser.EvaluationContext;
import se.fnord.ecalc.parser.EvaluationVisitor;
import se.fnord.ecalc.parser.ExpressionException;
import se.fnord.ecalc.parser.ExpressionParserContext;
import se.fnord.ecalc.parser.ExpressionParserVisitor;
import se.fnord.ecalc.parser.IFunction;
import se.fnord.ecalc.parser.IValue;
import se.fnord.ecalc.parser.Literal;
import se.fnord.ecalc.parser.OverlayContext;
import se.fnord.ecalc.parser.ParseException;
import se.fnord.ecalc.parser.SimpleNode;
import se.fnord.ecalc.parser.TokenMgrError;

public class UserFunction implements IFunction {

	private String[] paramnames;
	private SimpleNode expression;

	public UserFunction(String[] paramnames, SimpleNode expression) {
		this.paramnames = paramnames;
		this.expression = expression;
	}

	public UserFunction(String paramnames, SimpleNode expression) {
		this.paramnames = paramnames.split(",");
		this.expression = expression;
	}
	
	public String getParamNames() {
		StringBuffer sb = new StringBuffer();
		if (paramnames.length > 0) {
		
			sb.append(paramnames[0]);
			for (int i = 1; i < paramnames.length; i++)
				sb.append(",").append(paramnames[i]);
		}
		return sb.toString();
	}
	
	public SimpleNode getExpression() {
		return expression;
	}
	
	public IValue execute(IValue[] params, ExpressionParserContext context)
			throws ExpressionException {
		if (params == null || this.paramnames.length != params.length)
			throw new ExpressionException("%se.fnord.ecalc.exceptions.expressionexception.arity");
	
		try {
			OverlayContext function_context = new OverlayContext((EvaluationContext) context);
			for(int i = 0; i < params.length; i++) {
				function_context.setConstant(paramnames[i], new Literal(params[i]));
			}
			
			ExpressionParserVisitor visitor = new EvaluationVisitor();
			return expression.accept(visitor, function_context);

		} catch (ParseException exception) {
			throw new ExpressionException("%se.fnord.ecalc.exceptions.expressionexception.function_parse_error");
		} catch (TokenMgrError exception) {
			throw new ExpressionException("%se.fnord.ecalc.exceptions.expressionexception.function_parse_error");
		}
	}

}
